from lingres.conll import read_conll09_file
import sys

def extract_ud_coppola_constructions(ud_sent):
    ud_tree = ud_sent.dependency_tree

    constructions = set()

    for node in ud_tree:
        if node.incoming_edge_type == "cop":
            constructions.add((node.idx, node.parent.idx))

    return constructions

def check_is_tiger_modifier_modifier_pair(tiger_sent, idx_pair):
    dependency_tree = tiger_sent.dependency_tree

    if dependency_tree.nodes[idx_pair[1]].parent == dependency_tree.nodes[idx_pair[0]] and dependency_tree.nodes[idx_pair[1]].incoming_edge_type == "MO":
        return True
    else:
        return False

def extract_tiger_coppola_constructions(tiger_sent):
    tiger_tree = tiger_sent.dependency_tree

    constructions = set()

    for node in tiger_tree:
        if node.incoming_edge_type == "PD":
            constructions.add((node.parent.idx, node.idx))

    return constructions

def get_mismatching_pd_constructions(tiger_sent, ud_sent):
    ud_constructions = extract_ud_coppola_constructions(ud_sent)
    tiger_constructions = extract_tiger_coppola_constructions(tiger_sent)

    if ud_constructions != tiger_constructions:
        return ud_constructions - tiger_constructions, tiger_constructions - ud_constructions
    else:
        return False


if __name__ == "__main__":
    for sent_num, sents in enumerate(zip(read_conll09_file(sys.argv[1]), read_conll09_file(sys.argv[2]))):
        tiger_sent, ud_sent = sents
        mismatch_constructions = get_mismatching_pd_constructions(tiger_sent, ud_sent)
        if mismatch_constructions:
            ud_constructions, tiger_constructions = mismatch_constructions
            mod_constructions = set()
            for ud_mismatch in ud_constructions:
                if check_is_tiger_modifier_modifier_pair(tiger_sent, ud_mismatch):
                    mod_constructions.add(ud_mismatch)
            ud_constructions -= mod_constructions

            if ud_constructions != tiger_constructions:
                print ("Sentence", sent_num, "UD Construction", ud_constructions, "Tiger Construction", tiger_constructions)
                print (sent_num, ud_constructions, tiger_constructions)

