from lingres.conll import read_conll09_file, write_conll09_file
import sys

def find_coordination_chains_in_tree(tree):
    coordination_node_sets = []
    for node in tree.iter_token_nodes_left_to_right():
        if node.incoming_edge_type is not None and node.incoming_edge_type.startswith("CJ"):
            continue
        for child in node.children:
            if child.incoming_edge_type.startswith("CJ"):
                chain_nodes = extract_coordination_chain(child)
                coordination_node_sets.append((node.idx,) + tuple(chain_nodes))

    return coordination_node_sets

def extract_coordination_chain(node):
    coordination_nodes = [node.idx]
    curr_node = node
    while True:
        for child in curr_node.children:
            if child.incoming_edge_type.startswith("CJ"):
                coordination_nodes.append(child.idx)
                curr_node = child
                break
        else:
            break

    return coordination_nodes

def flatten_chain(tree, chain):
    root_node = tree.nodes[chain[0]]
    if chain[0] > chain[1]:
        print("Invert", chain, tree.nodes[chain[0]].token.form, tree.nodes[chain[1]].token.form)
    for idx in chain[1:]:
        tree.nodes[idx].parent = root_node

def convert_coordinations_in_tree(tree):
    for chain in find_coordination_chains_in_tree(tree):
        flatten_chain(tree, chain)

    return tree

def convert_tiger_trees(tiger_sents):
    for idx, tiger_sent in enumerate(tiger_sents):
        tiger_tree = convert_coordinations_in_tree(tiger_sent.dependency_tree)
        yield tiger_sent

if __name__ == "__main__":
    tiger_trees = read_conll09_file(sys.argv[1])

    write_conll09_file(sys.argv[2], convert_tiger_trees(tiger_trees))


