from lingres.conll import read_conll09_file, write_conll09_file
import argparse

def swap_parent_child_nodes(tree, parent_idx, child_idx, child_parent_label, parent_parent_child_label):
    parent_node = tree.nodes[parent_idx]
    child_node = tree.nodes[child_idx]

    parent_parent_node = parent_node.parent
    parent_node.remove_from_parent()

    if parent_parent_node is not None:
        parent_parent_node.add_child(child_node, parent_parent_child_label)
    else:
        child_node.remove_from_parent()
    child_node.add_child(parent_node, child_parent_label)


def find_nk_constructions_in_tiger_tree(tree):
    return find_parent_child_relations_with_label(tree, "NK")

def find_case_constructions_in_ud_tree(tree):
    return find_parent_child_relations_with_label(tree, "case", reverse_tuple_order = True)

def find_parent_child_relations_with_label(tree, label, reverse_tuple_order = False):
    constructions = set()
    for node in tree.iter_token_nodes_left_to_right():
        if node.incoming_edge_type == label:
            if reverse_tuple_order:
                result_tuple = (node.idx, node.parent.idx)
            else:
                result_tuple = (node.parent.idx, node.idx)
            constructions.add(result_tuple)

    return constructions

def transform_nk_to_case_constructions(tiger_sent, ud_sent):
    tiger_tree = tiger_sent.dependency_tree
    ud_tree = ud_sent.dependency_tree
    tiger_constructions = find_nk_constructions_in_tiger_tree(tiger_tree)
    ud_constructions = find_case_constructions_in_ud_tree(ud_tree)

    intersect_set = tiger_constructions.intersection(ud_constructions)

    #if not (len(intersect_set) == len(tiger_constructions) == len(ud_constructions)):
        #print ("Only in tiger:")
        #for it in (tiger_constructions - intersect_set):
        #    print (it)
        #print ("Only in ud:")
        #for it in (ud_constructions - intersect_set):
        #    print (it)
        #print()

    for it in intersect_set:
        parent_label = tiger_tree.nodes[it[0]].incoming_edge_type
        #if parent_label not in {"nmod", "dobj"}:
        #    print ("Found", parent_label, "in", tiger_sent.original_sentence_id)
        swap_parent_child_nodes(tiger_tree, it[0], it[1], "case", parent_label)

    return tiger_sent

def get_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("ud_trees")
    parser.add_argument("tiger_trees")
    parser.add_argument("outfile")

    return parser.parse_args()

if __name__ == "__main__":
    args = get_arguments()

    tiger_trees = read_conll09_file(args.tiger_trees)
    ud_trees = read_conll09_file(args.ud_trees)

    write_conll09_file(args.outfile, (transform_nk_to_case_constructions(tiger_tree, ud_tree) for tiger_tree, ud_tree in zip(tiger_trees, ud_trees)))

