from collections import defaultdict


class PyramidAlignmentReader:
    def __init__(self, filename):
        self.filename = filename

    def read_word_alignments(self):
        with open(self.filename) as f:
            for line in f:
                line_alignments = {}
                line = line.strip()
                for entry in line.split():
                    src_tok_idx, trg_tok_idx = map(int, entry.split("-"))
                    line_alignments.setdefault(src_tok_idx, []).append(trg_tok_idx)
               # if len(line_alignments) > 0:
                yield line_alignments

class AlignmentMerger:
    def __init__(self, source_sentences, target_sentences, alignments):
        self.source_sentences = source_sentences
        self.target_sentences = target_sentences
        self.alignments = alignments

    def read_sentences(self):
        idx = 0
        for source_sent, target_sent, alignments in zip(self.source_sentences, self.target_sentences, self.alignments):
            for tok in source_sent.tokens:
                tok.aligned_tokens = defaultdict(list)
            for tok in target_sent.tokens:
                tok.aligned_tokens = defaultdict(list)
            idx += 1
            for key, val in alignments.items():
                key_tok = source_sent[key]
                for trg_tok_idx in val:
                    trg_tok = target_sent[trg_tok_idx]
                    key_tok.aligned_tokens.setdefault("en", []).append(trg_tok)
                    trg_tok.aligned_tokens.setdefault("de", []).append(key_tok)

            yield source_sent, target_sent

