class MultiValueFieldDecoder:
    def __init__(self, num_values, seperator = ":", ambigous_sep_refitter = lambda values, desired_length, seperator: values):
        self.num_values = num_values
        self.seperator = seperator

        self.ambigous_sep_refitter = ambigous_sep_refitter

    def decode_sentences(self, sentences):
        for sent in sentences:
            yield self.decode_sentence(sent)

    def decode_sentence(self, sentence):
        new_sent_toks = []
        for tok in sentence:
            new_toks = [[] for _ in range(self.num_values)]
            for field in tok:
                values = field.split(self.seperator)

                if len(values) != 1 and len(values) != self.num_values:
                    values = self.ambigous_sep_refitter(values, self.num_values, self.seperator)
                    if values == None:
                        raise RuntimeError("Unexpected number of field values. Refitter could not decode at token: \n {}".format(tok))

                values = list(filter(lambda v: len(v) > 0, values))

                if len(values) == 1:
                    for idx in range(self.num_values):
                        new_toks[idx].append(values[0])
                elif len(values) == 0:
                    for idx in range(self.num_values):
                        new_toks[idx].append(field)
                elif len(values) == self.num_values:
                    for idx, val in enumerate(values):
                        new_toks[idx].append(val)
                else:
                    raise RuntimeError("Unexpected number of field values. Expected {} got {} at token: \n {}".format(len(values), self.num_values, tok))

            new_sent_toks.append(new_toks)

        return zip(*new_sent_toks)

