import sys

import numpy as np

from tabular import TabularReader, TabularWriter

def fold_sents(sents, num_folds = 5, randomize = False):
    indices = np.arange(len(sents))
    if randomize:
        np.random.shuffle(indices)
    
    fold_sizes = np.full(num_folds, len(sents) // num_folds)
    remainder = len(sents) / num_folds
    fold_sizes[0:remainder] += 1

    fold_mask = np.zeros(len(sents))
    fold_start = 0
    fold_end = fold_sizes[0] + 1

    for idx, fold_size in enumerate(fold_sizes):
        fold_mask[fold_start:fold_end] = idx

        if idx != num_folds -1:
            fold_start = fold_end
            fold_end += fold_sizes[idx + 1]

    fold_sents = [([], []) for _ in range(num_folds)]
    for sent_idx, sent_fold_idx in enumerate(fold_mask):
        for fold_idx in range(num_folds):
            if fold_idx == sent_fold_idx:
                fold_sents[fold_idx][1].append(sents[sent_idx])
            else:
                fold_sents[fold_idx][0].append(sents[sent_idx])

    return fold_sents

if __name__ == "__main__":
    reader = TabularReader(sys.argv[1])
    sents = list(reader.read_sentences())

    if len(sys.argv) <= 2:
        out_basename = sys.argv[1]
    else:
        out_basename = sys.argv[2]

    fold_idx = 0
    for train_fold_sentences, test_fold_sentences in fold_sents(sents):
        train_writer = TabularWriter(out_basename + ".fold." + str(fold_idx) + ".train")
        test_writer = TabularWriter(out_basename + ".fold." + str(fold_idx) + ".test")

        train_writer.write_sentences(train_fold_sentences)
        test_writer.write_sentences(test_fold_sentences)

        train_writer.close()
        test_writer.close()

        fold_idx += 1

