import sys

from lingres.tabular import TabularReader, TabularWriter

class TabularAnnotationOverlapChecker:
    def __init__(self, check_indices):
        self.check_indices = check_indices

    def check_rows(self, rows):
        values = dict([(ind, rows[0][ind]) for ind in self.check_indices])
        for row in rows[1:]:
            for ind in self.check_indices:
                if values[ind] != row[ind]:
                    return False
        return True

class TabularMerger:
    def __init__(self, indices):
        self.indices = indices

    def merge_sents(self, sents):
        merged_sent = []
        for rows in zip(*sents):
            merged_sent.append(
                self.merge_rows(rows)
            )
        return merged_sent

    def merge_rows(self, rows):
        t = TabularAnnotationOverlapChecker([1])
        new_row = []
        if not t.check_rows(rows):
            print("Mismatch", rows)
        for row_ind, val_ind in self.indices:
            new_row.append(rows[row_ind][val_ind])

        return new_row

def open_readers(filenames):
    readers = []
    for filename in filenames:
        readers.append(TabularReader(filename).read_sentences())
    return readers

def close_readers(readers):
    for reader in readers:
        reader.close()


def merge_tabular_main():
    filenames = []
    for filename in sys.argv[2:]:
        if filename == "--":
            break
        else:
            filenames.append(filename)

    indices = []
    for index_desc in sys.argv[3 + len(filenames):]:
        file_idx, val_idx = index_desc.split(":")

        file_idx = int(file_idx)
        
        val_indices = val_idx.split("-")
        if len(val_indices) == 2:
            val_indices = list(range(int(val_indices[0]), int(val_indices[1]) + 1))
        else:
            val_indices = [int(val_idx)]

        for val_idx in val_indices:
            indices.append((file_idx, val_idx))

    merger = TabularMerger(indices)

    readers = open_readers(filenames)
    writer = TabularWriter(sys.argv[1])

    sent_cnt = 0
    for sents in zip(*readers):
        sent_cnt += 1
        merged_sent = merger.merge_sents(sents)
        writer.write_sentence(merged_sent)

    print ("Processed {} sentences".format(sent_cnt))
    
    close_readers(readers)
    writer.close()

if __name__ == "__main__":
    merge_tabular_main()
