class TabularReader:
    def __init__(self, filename, column_count = None):
        self.filename = filename
        self.column_count = column_count

    def read_sentences(self):
        with open(self.filename) as f:
            curr_sent = []
            for line in f:
                line = line.strip()
                if line == "":
                    yield curr_sent
                    curr_sent = []
                else:
                    columns = line.split()
                    if not self.column_count:
                        self.column_count = len(columns)
                    if self.column_count == len(columns):
                        curr_sent.append(columns)
                    elif self.column_count - 1 == len(columns):
                        line = line.replace("\t" * 2, "\t--\t")
                        columns = line.split()
                        curr_sent.append(columns)
                    elif self.column_count - 2 == len(columns):
                        line = line.replace("\t" * 3, "\t--\t--\t")
                        columns = line.split()
                        curr_sent.append(columns)
                    else:
                        raise RuntimeError("Badly formatted line '{}', wrong column count (expected {}, got {})".format(line, self.column_count, len(columns)))

            if len(curr_sent) > 0:
                yield curr_sent

def write_tabular_sent_to_stream(sent, stream):
    for tok in sent:
        stream.write("\t".join(tok))
        stream.write("\n")
    stream.write("\n")

class TabularWriter:
    def __init__(self, filename):
        self.filename = filename
        self.is_open = False

    def write_sentence(self, sent):
        if not self.is_open:
            self.open()
        for tok in sent:
            self.file_handle.write("\t".join(tok))
            self.file_handle.write("\n")
        self.file_handle.write("\n")

    def write_sentences(self, sents):
        for sent in sents:
            self.write_sentence(sent)

    def open(self):
        self.file_handle = open(self.filename, "w")
        self.is_open = True

    def close(self):
        self.file_handle.close()
        self.is_open = False

